#include "stdafx.h"
#include "dataset.h"
#include "stdio.h"
#include "utils.h"
#include <math.h>



void Dataset::loadFromFile(char* fn){

FILE* f;
f = fopen(fn,"r");
if(f==NULL) { printf("File not found: %s.\n",fn);}else{
fscanf(f,"%i %i",&pointCount,&coordCount);
float *vec;
mas = new float*[pointCount];
for(int i=0;i<pointCount;i++){
	vec = new float[coordCount];
	for(int j=0;j<coordCount;j++){
		fscanf(f,"%f",&vec[j]);
		//printf("%f ",vec[j]);
	}
	mas[i] = vec;
	//for(j=0;j<coordCount;j++){
	//	printf("%f ",vec[j]);
	//}
	//printf("\n");
}
fclose(f);
}
};

void Dataset::loadWeights(char* fn){
FILE* f;
f = fopen(fn,"r");
weighted = true;
weightSum = 0;
if(f==NULL) { printf("File not found.\n");}else{
weights = new float[pointCount];
for(int i=0;i<pointCount;i++){
		fscanf(f,"%f",&weights[i]);
		weightSum+=weights[i];
	}
fclose(f);
}

}

void Dataset::saveToFile(char* fn){
FILE* f;
f = fopen(fn,"w");

fprintf(f,"%i %i\n",pointCount,coordCount);
for(int i=0;i<pointCount;i++){
	float *vec = getVector(i);
	for(int j=0;j<coordCount;j++)
		fprintf(f,"%10.3f ",vec[j]);
	fprintf(f,"\n");
}
fclose(f);
};

void Dataset::init(int points, int dim){

};

void Dataset::allocate(){
	mas = new float*[pointCount];
	for(int i=0;i<pointCount;i++){
		mas[i] = new float[coordCount];
	}
	allocated = true;
};

void Dataset::divideRandomly(Dataset *ds1,Dataset *ds2,float percentage){
	//randomize();
	bool *sel = new bool[pointCount];
	int n = 0;
	for(int i=0;i<pointCount;i++){
		float r = (float)rand()/RAND_MAX;
		//printf("r = %f\n",r);
		if(r<percentage) { sel[i] = true; n++; } else { sel[i] = false;  }
	}

	ds1->pointCount = pointCount - n;
	ds2->pointCount = n;
	ds1->coordCount = coordCount;
	ds2->coordCount = coordCount;
	ds1->normToZscores = normToZscores;
	ds2->normToZscores = normToZscores;
	ds1->weighted = weighted;
	ds2->weighted = weighted;
	ds1->allocate();
	ds2->allocate();
	ds1->weights = new float[ds1->pointCount];
	ds2->weights = new float[ds2->pointCount];


	int n1 = 0, n2 = 0;
	for(i=0;i<pointCount;i++){
		//printf("sel = %i\n",sel[i]);
		if(!sel[i]){
			for(int k=0;k<coordCount;k++)
				ds1->mas[n1][k] = mas[i][k];
			if(ds1->weighted){
				ds1->weights[n1] = weights[i];
				ds1->weightSum+=weights[i];
			}
			n1++;
			//printf("n1 = %i\n",n1);
		}else{
			for(int k=0;k<coordCount;k++)
				ds2->mas[n2][k] = mas[i][k];
			if(ds2->weighted){
				ds2->weights[n2] = weights[i];
				ds2->weightSum+=weights[i];
			}
			n2++;
			//printf("n2 = %i\n",n2);
		}
	}

	delete [] sel;
};

float* Dataset::getVectorUnNormalized(int i){
	float *v;
	v = new float[coordCount];
	for(int j=0;j<coordCount;j++) v[j] = mas[i][j];
	unnorm(v);
	return v;
};

float* Dataset::getVector(int i){
	return mas[i];
};


void Dataset::normalize(){
	for(int i=0;i<pointCount;i++){
		float *vec = getVector(i);
		norm(vec);
	}
};

void Dataset::norm(float *vec){
	if(stat!=NULL){
		for(int i=0;i<coordCount;i++){
			if(normToZscores){
				if(stat->StdDevs[i]!=0)
					vec[i] = (float)(((double)vec[i] - stat->Means[i])/stat->StdDevs[i]);
				else
					vec[i] = (float)((double)vec[i] - stat->Means[i]);
			}
		}
	}
};

void Dataset::unnorm(float *vec){
	if(stat!=NULL){
		for(int i=0;i<coordCount;i++){
			if(normToZscores){
				vec[i] = (float)((double)vec[i]*stat->StdDevs[i]+stat->Means[i]);
			}
		}
	}
};

// -------------------- Statistics ----------------------------------


void Statistics::calculate(Dataset *d){

	Means = new double[d->coordCount];
	StdDevs = new double[d->coordCount];
	Mins = new double[d->coordCount];
	Maxs = new double[d->coordCount];
	for(int i=0;i<d->coordCount;i++){
		Mins[i] = +1e10;
		Maxs[i] = -1e10;
		Means[i] = 0;
		StdDevs[i] = 0;
	}

	//for(int jj=0;jj<d->coordCount;jj++)
	//	printf("%5.2f ",StdDevs[jj]); printf("\n");


	float *vec;
	float n = 0;
	for(i=0;i<d->pointCount;i++){
		vec = d->getVector(i);
		if(!d->weighted) n+=(float)1; else n+=d->weights[i];
		for(int j=0;j<d->coordCount;j++){
			if(!d->weighted){
				Means[j]+=vec[j];
			}else{
				Means[j]+=vec[j]*d->weights[i];
			}
			if(!d->weighted){
				StdDevs[j]+=(double)(vec[j]*vec[j]); 
			}else{
				StdDevs[j]+=(double)(vec[j]*vec[j]*d->weights[i]); 
			}
			if(vec[j]<Mins[j]) Mins[j] = vec[j];
			if(vec[j]>Maxs[j]) Maxs[j] = vec[j];
			//printf("%4.2f ",vec[j]);
		};
		//printf("\n");
	};

	//for(jj=0;jj<d->coordCount;jj++)
	//	printf("%5.2f ",StdDevs[jj]);

	totalDispersion = 0;
	for(int j=0;j<d->coordCount;j++){
		Means[j]/=n;
		StdDevs[j]=StdDevs[j]/n-Means[j]*Means[j];
		totalDispersion+=StdDevs[j];
		StdDevs[j]=sqrt(StdDevs[j]);
	};
	/*double s = 0;
	double s2 = 0;
	double dd = 0;
	for(j=0;j<d->pointCount;j++){
		dd = squareEuclDistance(Means,d->getVector(j),d->coordCount);
		s+=dd;
		s2+=dd*dd;
	}
	s/=d->pointCount;
	totalDispersion = s;*/
	//totalDispersion = totalDispersion;
};
