#include "stdafx.h"
#include <stdlib.h>
#include <time.h>
#include "grid.h"
#include "dataset.h"
#include "princomp.h"
#include "elmapalg.h"
#include "slausolver.h"
#include "utils.h"

int main(int argc, char* argv[])
{

	char *inifile = NULL;
	char *datfile = NULL;
	char *outfile = NULL;
	char *wfile = NULL;
	char *genmanif = NULL;
	int algnum = -1;
	char *dat = "none";
	int testperc = -1;
	int normalize = 1;

	for(int i=0;i<argc;i++){
		if(strcmp(argv[i],"-d")==0) datfile = argv[i+1];
		if(strcmp(argv[i],"-i")==0) inifile = argv[i+1];
		if(strcmp(argv[i],"-n")==0) algnum = atoi(argv[i+1]);
		if(strcmp(argv[i],"-o")==0) outfile = argv[i+1];
		if(strcmp(argv[i],"-w")==0) wfile = argv[i+1];
		if(strcmp(argv[i],"-dat")==0) dat = argv[i+1];
		if(strcmp(argv[i],"-t")==0) testperc = atoi(argv[i+1]);
		if(strcmp(argv[i],"-norm")==0) normalize = atoi(argv[i+1]);
		if(strcmp(argv[i],"-gen")==0) genmanif = argv[i+1];
	}

	if(datfile!=NULL){
		printf("Dat file: %s\n",datfile);
	}else{ printf("FATAL ERROR: No data file was specified.\n"); exit(-1); };
	if(wfile!=NULL)
		printf("Weight file: %s\n",wfile);
	if(genmanif!=NULL)
		printf("Generating manifold file: %s\n",genmanif);
	if(inifile!=NULL){
		printf("Ini file: %s\n",inifile);
	}else{ printf("FATAL ERROR: No initialization file was specified.\n"); exit(-1); };
	if(inifile!=NULL){
		printf("Output file: %s\n",outfile);
	}else{ printf("FATAL ERROR: No output file was specified.\n"); exit(-1); };
	if(algnum>=0){
		printf("Alg number: %i\n",algnum);
	}else{ printf("FATAL ERROR: algorithm number must be specified.\n"); exit(-1); };

	Dataset ds;
	ds.loadFromFile(datfile);
	Dataset d;
	Dataset dtest;
	Dataset *data = &ds;
	Statistics s;
	s.calculate(&ds);
	ds.stat = &s;
	if(normalize==1)
	    ds.normalize();
	if(wfile!=NULL) ds.loadWeights(wfile);

	if(testperc>=0){
		ds.divideRandomly(&d,&dtest,(float)testperc/100);
		d.stat = &s;
		data = &d;
	}

	PCAMethod pc;
	pc.ds = data;
	long ts = clock();
	pc.calc(3);
	printf("Time spent for calc PC: %i\n",clock()-ts);
	pc.calcDispersions();
	pc.saveToFile("temp");

	ElasticMapsAlgorithm elmap;
	elmap.setData(data);
	if(testperc>=0){
		elmap.setDataTest(&dtest);
	}

	elmap.readIniFile("elmap.ini",algnum);

	if(genmanif!=NULL){
		Grid *gr = new Grid();
		gr->dimension = elmap.grid->dimension;
		gr->loadNodesFromFile(genmanif);
		elmap.setGenManif(gr);
	}

	SLAUSolver solv;
	elmap.setSolver(&solv);
	long cl = clock();
	elmap.doGraphConstruct();
	//elmap.doit();

	cout << "Time spent: " << clock()-cl << endl;
	elmap.grid->saveToFile(outfile,dat);
	
	return 0;
}

