#include "stdafx.h"
#include <stdlib.h>
#include <time.h>
#include "grid.h"
#include "dataset.h"
#include "princomp.h"
#include "elmapalg.h"
#include "slausolver.h"
#include "utils.h"

int main(int argc, char* argv[])
{

	char *inifile = NULL;
	char *datfile = NULL;
	char *outfile = NULL;
	char *wfile = NULL;
	char *genmanif = NULL;
	char *projfile = NULL;
	char *normout = NULL;
	char *pcout = NULL;
	int algnum = -1;
	char *dat = "none";
	int testperc = -1;
	int takeperc = -1;
	int normalize = 1;

	for(int i=0;i<argc;i++){
		if(strcmp(argv[i],"-d")==0) datfile = argv[i+1];
		if(strcmp(argv[i],"-i")==0) inifile = argv[i+1];
		if(strcmp(argv[i],"-n")==0) algnum = atoi(argv[i+1]);
		if(strcmp(argv[i],"-o")==0) outfile = argv[i+1];
		if(strcmp(argv[i],"-w")==0) wfile = argv[i+1];
		if(strcmp(argv[i],"-dat")==0) dat = argv[i+1];
		if(strcmp(argv[i],"-t")==0) testperc = atoi(argv[i+1]);
		if(strcmp(argv[i],"-take")==0) takeperc = atoi(argv[i+1]);
		if(strcmp(argv[i],"-norm")==0) normalize = atoi(argv[i+1]);
		if(strcmp(argv[i],"-normdat")==0) normout = argv[i+1];
		if(strcmp(argv[i],"-pcdat")==0) pcout = argv[i+1];
		if(strcmp(argv[i],"-gen")==0) genmanif = argv[i+1];
		if(strcmp(argv[i],"-p")==0) projfile= argv[i+1];
	}

	if(datfile!=NULL){
		printf("Dat file: %s\n",datfile);
	}else{ printf("FATAL ERROR: No data file was specified.\n"); exit(-1); };
	if(wfile!=NULL)
		printf("Weight file: %s\n",wfile);
	if(genmanif!=NULL)
		printf("Generating manifold file: %s\n",genmanif);
	if(inifile!=NULL){
		printf("Ini file: %s\n",inifile);
	}else{ printf("FATAL ERROR: No initialization file was specified.\n"); exit(-1); };
	if(inifile!=NULL){
		printf("Output file: %s\n",outfile);
	}else{ printf("FATAL ERROR: No output file was specified.\n"); exit(-1); };
	if(algnum>=0){
		printf("Alg number: %i\n",algnum);
	}else{ printf("FATAL ERROR: algorithm number must be specified.\n"); exit(-1); };
	if(projfile!=NULL)
		printf("File for projections: %s\n",projfile);


	Dataset ds;
	ds.loadFromFile(datfile);
	Dataset d;
	Dataset dtest;
	Dataset *data = &ds;
	Statistics s;
	s.calculate(&ds);
	ds.stat = &s;
	if(normalize==1)
	    ds.normalize();
	if(normout!=NULL)
		d.saveToFile(normout);
	if(wfile!=NULL) ds.loadWeights(wfile);

	if(testperc>=0){
		ds.divideRandomly(&d,&dtest,(float)testperc/100);
		d.stat = &s;
		data = &d;
	}

	if(takeperc>=0){
		ds.divideRandomly(&d,&dtest,(float)takeperc/100);
		d.stat = &s;
		data = &d;
	}


	PCAMethod pc;
	pc.ds = data;
	long ts = clock();
	pc.calc(3);
	printf("Time spent for calc PC: %i\n",clock()-ts);
	pc.calcDispersions();
	pc.saveToFile("temp");

	//pc.saveProjections("pcpr25.txt");

	if(pcout!=NULL){
		pc.saveProjections(pcout);
	}

	ElasticMapsAlgorithm elmap;
	elmap.setData(data);
	if(testperc>=0){
		elmap.setDataTest(&dtest);
	}

	elmap.readIniFile("elmap.ini",algnum);

	if(genmanif!=NULL){
		Grid *gr = new Grid();
		gr->dimension = elmap.grid->dimension;
		gr->loadNodesFromFile(genmanif);
		elmap.setGenManif(gr);
	}

	//elmap.setGrid(&gr);
	SLAUSolver solv;
	elmap.setSolver(&solv);
	elmap.initializeGrid(pc.Shift,pc.comps,pc.dispersions,3);

	/*elmap.calcTaxons();
	elmap.calcSystemMatrix("");
	elmap.solver->saveMatrix("temp.txt");
	double *vec = new double[solv.dimension];
	elmap.calcRightHandVector(0,vec);
	elmap.solver->setVector(vec);
	//elmap.solver->saveVector("temp.txt");*/

	/*double *sol = new double[solv.dimension];
	for(int i=0;i<gr.nodesNum;i++){
		sol[i] = gr.Nodes[i][0];
	}
	solv.setSolution(sol);
	solv.saveSolution("sol.txt");*/
	
	//elmap.doTheEpochStrategy();
	cout << "Proceeding with " << elmap.data->pointCount << " points" << endl;
	long cl = clock();
	if(elmap.grid->type!=4)
		elmap.doit();
	else{
		elmap.doGraphConstruct();
		//ElasticGraph *gr = (ElasticGraph*)elmap.grid;
		//gr->test(elmap.data);
	}

	/*for(i=0;i<50;i++){
	if(i==3)
		cout << "Adaptation: " << i+1 << endl;
	elmap.grid->adaptate(elmap.taxons); 
	elmap.currentEpoch--; 
	elmap.proceedToTheNextEpoch();
	}*/

	/*for(i=0;i<150;i++){
		cout << "Adaptation step " << (i+1) << endl;
		elmap.grid->adaptate(elmap.taxons); 
		elmap.calcTaxons();
		elmap.currentEpoch--; 
		elmap.proceedToTheNextEpoch();
	}*/

	cout << "Time spent: " << clock()-cl << endl;
	elmap.calcTaxons();
	cout << "Final MSE: " << elmap.calcMSE() << endl;

	elmap.grid->saveToFile(outfile,dat);

	if(projfile!=NULL){
		elmap.grid->saveProjections(projfile,elmap.data);
	}

	if(normout!=NULL){
		FILE *f;
		f = fopen(normout,"w");
		if(f==NULL) { printf("Can not open file %s for writing. \n",normout); exit(1); }
		for(int i=0;i<elmap.data->pointCount;i++){
			for(int j=0;j<elmap.data->coordCount;j++){
				fprintf(f,"%10.10f\t",elmap.data->mas[i][j]);
			}
        fprintf(f,"\n");
		}
		fclose(f);
	}

	/*double *mas = new double[elmap.grid->nodesNum];
	FILE *f;
	f = fopen("dist_nodes","w");

	vector<vector<int> > taxons;
	elmap.grid->calcTaxons(taxons,elmap.data);
	elmap.grid->getDistancesToCentroids(mas,elmap.data,taxons);

	double mean = 0,std = 0;

	calcSimpleStatistics(mas,elmap.grid->nodesNum,mean,std);
	printf("Mean = %f, Std = %f\n",mean,std);
	for(int ii=0;ii<elmap.grid->nodesNum;ii++){
		fprintf(f,"%10.10f\n",mas[ii]);		
	}
	fclose(f);
	delete [] mas;*/

	//LinearGrid *lgr = (LinearGrid *)elmap.grid;
	//lgr->saveInNaturalOrder("naturalord.txt");
	
	return 0;
}

