#include "stdafx.h"
#include "vector.h"

class SimpleRectangularGrid;
class Dataset;
class Extendable;

class Grid{

public:

int nodesNum;
int edgesNum;
int ribsNum;
int trianglesNum;

/*float** Nodes;
float** Nodes3D;
int** Edges;
float *EP;
int** Ribs;
float *RP;
int** Triangles;*/

char name[1000];
vector<vector<float> > Nodes;
vector<vector<float> > Nodes3D;
vector<vector<int> > Edges;
vector<int> nodesIndex;
vector<float> EP;
vector<vector<int> > Ribs;
vector<float> RP;
vector<vector<int> > Triangles;
vector<vector<int> > connectList;
float *NodesCopy;
float **NodesCopy2;
int n2;

int type; // 0 - 2D rectangular
int dimension;
int intDim;

Grid(){
	NodesCopy = NULL;
	NodesCopy2 = NULL;
	//Nodes = Nodes3D = NULL;
	//Edges = Ribs = Triangles = NULL;
	sprintf(name,"Noname");
}
~Grid(){
	//printf("In grid destructor: %sm type %i\n",name,type);
	free();
}

void loadFromFile(char* fn); 
void saveToFile(char* fn,char *dat); 
virtual void generate();
void allocate();
void free();
void copyfrom(Grid &);
void extrapolate();
double calcMSE(Dataset *d, vector<vector<int> > taxons);
double MSEDistanceTo(Grid *g,int sil);
void MakeNodesCopy();

void loadNodesFromFile(char *fn);
void calcTaxons(vector<vector<int> > &v, Dataset *d);
virtual void adaptateGrow(vector<vector<int> >,int degree);
virtual void adaptateBreak(vector<vector<int> >,Dataset *dat);
void divideEdgeInHalf(int num, int adapt);
void projectPoint(float *point, float *res);
void saveProjections(char *fn, Dataset *ds);

void getDistancesToCentroids(double *mas, Dataset *dat, vector<vector<int> > taxons);
void makeConnectList();
void connectGraph();

void fixRElasticity(int mode);
void fixEElasticity(int mode);

};

class Extendable {
public:
	Extendable(){
	};
	void extrapolate(); 
};


class SimpleRectangularGrid: public Grid{

public:
	int rowCount;
	int colCount;
	SimpleRectangularGrid(int cols, int rows, int dim);
	//~SimpleRectangularGrid();

	void generate();
	void extrapolate();
	void interpolate();
};


class SphericalGrid: public Grid{

public:
	int rowCount;
	int colCount;
	SphericalGrid(int cols, int rows, int dim);
	void generate();
};

class LinearGrid: public Grid, Extendable{

public:
	int pointCount;
	LinearGrid(int points, int dim);
	void generate();
	void extrapolate();
	void interpolate();
	void saveInNaturalOrder(char *fn);
	//virtual void adaptate(vector<vector<int> > taxons, int degree);
};

class CircleGrid: public Grid{

public:
	int pointCount;
	CircleGrid(int points, int dim);
	void generate();
};

class ElasticGraph: public Grid{
public:
	int pointCount;
	int minNumInCluster;
	vector<Grid> subgrids;
	ElasticGraph(int points, int dim);
	void generate();
	void KMeansInitialization(Dataset *d, int numiter, double eps);
	void localDipoles(Dataset *d);
	void synchronize();
	void test(Dataset *d);
};


