#include "stdafx.h"
#include "dataset.h"
#include "princomp.h"
#include "utils.h"
#include <math.h>
#include <time.h>
#include <minmax.h>

PCAMethod::PCAMethod(){
	dispersions = NULL;
	silent = false;
}

PCAMethod::~PCAMethod(){
	if(dispersions!=NULL) delete [] dispersions;
}


void PCAMethod::calc(int numcomp){

numberOfComps = numcomp;

comps = new double*[numcomp];
dispersions = new double[numberOfComps];

int dimen = ds->coordCount;
int pointNum = ds->pointCount;
//printf("pointNum = %i\n",pointNum);
float **dDat;
dDat = new float*[dimen];
for(int i=0;i<dimen;i++) dDat[i] = new float[pointNum];

double *Comp;
Comp = new double[dimen];

Shift = new double[dimen];
for(i=0;i<dimen;i++){ 
	double sh = 0, sw = 0;
	for(int j=0;j<ds->pointCount;j++){
		if(ds->weighted){
			sh+=ds->mas[j][i]*ds->weights[j];
			sw+=ds->weights[j];
		}else{
			sh+=ds->mas[j][i];
			sw+=1;
		}
	}
	Shift[i]=sh/sw;
}

for(int j=0;j<pointNum;j++)
for(i=0;i<dimen;i++) dDat[i][j]=ds->mas[j][i]-Shift[i];

for(i=0; i<numcomp; i++){
randomVector(Comp,dimen);
//for(int k=0;k<5;k++) printf("%5.2f ",Comp[k]); printf("\n");
if(!silent)
	printf("Comp %i \n",i); 
calculateFirstPC(dDat,Comp,Shift);
//for(k=0;k<5;k++) printf("%5.3f ",Comp[k]); printf("\n");


double d = vectorNorm(Comp,dimen);
for(int k=0;k<dimen;k++) Comp[k]/=d;
comps[i] = new double[dimen];
for(k=0;k<dimen;k++) comps[i][k] = (float)Comp[k];

double st = 0, st2 = 0;
double t=0;
double n = 0;

//if(silent)
//		for(int kk=0;kk<pointNum;kk++){
//			printf("%f %f \n",dDat[0][kk]-Shift[0],dDat[1][kk]-Shift[1]);
//		}


for(j=0; j<pointNum; j++){
  if(!ds->weighted) n+=1; else n+=ds->weights[j];
  t=0;
  for(k=0; k<dimen; k++) t+=(dDat[k][j])*Comp[k];
  //if(silent)
  //	 printf("%f ",t);
  for(k=0; k<dimen; k++) dDat[k][j] = (float)(dDat[k][j]-t*Comp[k]);
  if(!ds->weighted){
	st+=t;
	st2+=t*t;
  }else{
	st+=t*ds->weights[j];
	st2+=t*t*ds->weights[j];
  }
  //if(silent)
  //	printf("\n");
  //if(ds->weights[j]>1){
  //	  printf("%i : t = %f \n",j,t);
  //}
 }

st/=n;
st2 = st2/n - st*st;
dispersions[i] = st2;

}

//printf("\n");

for(int k=0;k<dimen;k++){ 
	double sh = 0, sw = 0;
	for(int j=0;j<ds->pointCount;j++){
		if(ds->weighted){
			sh+=ds->mas[j][k]*ds->weights[j];
			sw+=ds->weights[j];
		}else{
			sh+=ds->mas[j][k];
			sw+=1;
		}
	}
	Shift[k]=sh/sw;
}


for(i=0;i<dimen;i++) delete [] dDat[i];
delete [] dDat;
delete [] Comp;

};

void PCAMethod::calculateFirstPC(float** dDat, double* Vector, double* Shift){
double eps = EPSILON;

int dimen = ds->coordCount;
int pointNum = ds->pointCount;

double *MY = new double[dimen];
double *MB = new double[dimen];
double *Ord1 = new double[dimen];

for (int i=0; i<dimen; i++)
 {
 MY[i]=Vector[i];
 MB[i]=Shift[i];
 }
int j=0;
double e1=MAX_VALUE, e2=MAX_VALUE, e=MAX_VALUE;
double sy=0,sb=0;
while (e>eps)
  {
  //for (int k=0; k<5; k++) printf("%5.2f ",Vector[k]); printf("\n");
  //long cl = clock();
  PCIteration(dDat,Vector,Shift);
  //printf("%i\n",clock()-cl);
  e1=0; e2=0; e = 0; 
  for(i=0;i<dimen;i++){
  sy+=Vector[i]*Vector[i];   sb+=Shift[i]*Shift[i];
  e1 = max(fabs(MY[i]-Vector[i]),e1);
  e2 = max(fabs(MB[i]-Shift[i]),e2);
  }
  e = max(e1,e2);
  //printf("%f \n",e);
  for (i=0; i<dimen; i++)
    {
    MY[i]=Vector[i];
    MB[i]=Shift[i];
    }
  j++;
  if (j>MAX_NUM_ITER){
     e1=0;
     e2=0;
     e=0;
	 if(!silent)
        printf("The process did not converge\n");
     }
  }

Ord1[0]=1;
double sp=0;
for (i=0; i<dimen; i++) sp+=Vector[i]*Ord1[i];
if (sp<0) for (i=0; i<dimen; i++) Vector[i]*=-1;

if(!silent)
  printf("%i iterations\n",j);

delete [] MY;
delete [] MB;
delete [] Ord1;

}


void PCAMethod::PCIteration(float **dDat, double *Vector, double *Shift){
int dimen = ds->coordCount; 
int pointNum = ds->pointCount;
double *X;
X = new double[pointNum];
double *V = new double[pointNum];
double s1=0, s2=0;


  s2=vectorNorm(Vector,dimen);
  s2 = s2*s2;
  for (int i=0; i<pointNum; i++)
  {
  s1=0; //s2=0;
  for (int j=0;j<dimen; j++)
     s1=s1+(dDat[j][i]-Shift[j])*Vector[j];
  X[i]=s1/s2;
  }
//long cl = clock();

double a11=0, a12=0, a21=0, a22=0, b1=0, b2=0;
	for (int j=0;j<dimen;j++)
	{
	double d = 0;
	int i = 0;
if(!ds->weighted){
	a11=0, a12=0, a21=0, b1=0, b2=0;
	a22 = ds->pointCount;
	for (i=0; i<pointNum; i++)
	    {
		a11+=X[i]*X[i];
	    a12+=X[i];
		b1+=dDat[j][i]*X[i];
		b2+=dDat[j][i];
		}
}else{
	a11=0, a12=0, a21=0, a22=0, b1=0, b2=0;
	for (i=0; i<pointNum; i++)
	    {
		a11+=X[i]*X[i]*ds->weights[i];
	    a12+=X[i]*ds->weights[i];
		a22+=ds->weights[i];
		b1+=dDat[j][i]*X[i]*ds->weights[i];
		b2+=dDat[j][i]*ds->weights[i];
		}
}
    
  a21=a12;
  double det=a11*a22-a12*a21;
  double det1=b1*a22-b2*a12;
  double det2=a11*b2-a21*b1;
  Vector[j]=det1/det;
  Shift[j]=det2/det;
  }
//printf("%i\n",clock()-cl);
  double d = vectorNorm(Vector,dimen);;
  for(i=0;i<dimen;i++) Vector[i]/=d;
delete [] X;
delete [] V;
}


void PCAMethod::saveToFile(char *fn){
FILE* f;
f = fopen(fn,"w");
if(f==NULL) { printf("Can not use %s !!!\n",fn); }
else{

fprintf(f,"%i %i\n",numberOfComps,ds->coordCount);
for(int j=0;j<ds->coordCount;j++)
	fprintf(f,"%10.3f ",Shift[j]);
	fprintf(f,"\n");

for(int i=0;i<numberOfComps;i++){
	double *vec = comps[i];
	for(int j=0;j<ds->coordCount;j++)
		fprintf(f,"%10.5f ",vec[j]);
	fprintf(f,"\n");
}

/*for(i=0;i<numberOfComps;i++){
	float *vec = comps[i];
	for(j=i;j<numberOfComps;j++){
		double d =scalarProduct(vec,comps[j],ds->coordCount);
		fprintf(f,"%10.5f ",d);
	}
	fprintf(f,"\n");
}*/

double s=0;
if(dispersions!=NULL)
for(int i=0;i<numberOfComps;i++){
	s+=dispersions[i];
	if(dispersions[i]<1e-3)
		fprintf(f,"%e\t%f\n",dispersions[i],s);
	else
		fprintf(f,"%f\t%f\n",dispersions[i],s);
}
fclose(f);
}
}

void PCAMethod::calcDispersions(){
	//printf("Pc.ds->pointCount %i\n",ds->pointCount);
	Statistics stat;
	stat.calculate(ds);
    for(int i=0;i<numberOfComps;i++){
	dispersions[i]/=stat.totalDispersion;
	}
    
/*	float *v;
	float *sh = new float[ds->coordCount];
	float *sv = new float[ds->coordCount];
	for(int i=0;i<ds->coordCount;i++) sh[i] = (float)(Shift[i]);
	for(i=0;i<numberOfComps;i++){
		v = comps[i];
		double d = 0;
		double d2 = 0;
		for(int j=0;j<ds->pointCount;j++){
			float *vec = ds->getVector(j);
			subtract(vec,sh,sv,ds->coordCount);
			double t = scalarProduct(sv,v,ds->coordCount);
			d+=t; d2+=t*t;
			//if(i==0)
			//printf("%f\n",t);
		}
		//float ss = vectorNorm(v,ds->coordCount);
		//printf("%f ",ss);
		d/=ds->pointCount;
		d2 = d2/ds->pointCount - d*d;
		dispersions[i] = sqrt(d2)/stat.totalDispersion;
		//printf("%f ",dispersions[i]);
	}
	delete [] sh;
	delete [] sv;*/
}

void PCAMethod::saveProjections(char *fn){
	FILE *f;
	f = fopen(fn,"w");
	if(f==NULL) { printf("Can not open file for writing %s",fn); }
	else{
		for(int i=0;i<ds->pointCount;i++){
			for(int j=0;j<this->numberOfComps;j++){
				float s = 0;
				for(int k=0;k<ds->coordCount;k++) s+=(ds->mas[i][k]-Shift[k])*comps[j][k];
				fprintf(f,"%10.10f\t",s);
			}
		fprintf(f,"\n");
		}
		fclose(f);
	}
}