#include "stdafx.h"
#include "slausolver.h"
#include <time.h>

#include <stdlib.h>
#include <iostream.h>

#include "compcol_double.h"
#include "coord_double.h"
#include "iohb_double.h"
#include "iotext_double.h"
#include "mvblasd.h"
#include "icpre_double.h"

#include "cg.h"
//#include "sparseadpt.h"

// implementation of SLAUSolver adapter, using SparseLib and IML++

void SLAUSolver::readMatrix(char *fn){
	readHB_mat(fn,Matrix);
}

void SLAUSolver::saveMatrix(char *fn){
	//writeHB_mat(fn,*Matrix);
	writetxtfile_mat(fn,*Matrix);
}

void SLAUSolver::readVector(char *fn){
	readHB_rhs(fn,vector_v);
}

void SLAUSolver::saveVector(char *fn){
	double *vector_d = new double[dimension];
	for(int i=0;i<dimension;i++){
		vector_d[i] = vector[i];
	}
	writeHB_mat(fn,*Matrix,1,vector_d);
	delete [] vector_d;
}

void SLAUSolver::saveSolution(char *fn){
	FILE *f;
	f = fopen(fn,"w");
	for(int i=0;i<dimension;i++){
		fprintf(f,"%10.8f\n",solution[i]);
	}
	fclose(f);
}


void SLAUSolver::setVector(double *vec){
	//for(int i=0;i<dimension;i++) cout << vec[i] << ","; cout << endl;
	vector = vec;
	vector_v = new MV_Vector_double(dimension);
	for(int i=0;i<dimension;i++){ //cout << vec[i] << ",";
		(*vector_v)[i] = vec[i]; 
	}
}


void SLAUSolver::setSolution(double *sol){
	solution = sol;
	solution_v = new MV_Vector_double(dimension);
	for(int i=0;i<dimension;i++)
		(*solution_v)[i] = sol[i];
}

long SLAUSolver::solve(double tolerance, int maxNumberOfIter){
	long cl = clock();
	int result = 0;
	//cout << "Before: " ;
	//for(int i=0;i<dimension;i++) cout << (*solution_v)[i] << " "; 
	//cout << endl;
	result = CG(*Matrix,*solution_v,*vector_v,*Preconditioner,maxNumberOfIter,tolerance);
	if(result!=0) 
	   printf("Result = %i , tol = %8.5f, iter = %i\n",result,tolerance,maxNumberOfIter);
	for(int i=0;i<dimension;i++) solution[i] = (*solution_v)[i];
	//cout << "After: " ;
	//for( i=0;i<dimension;i++) cout << (*solution_v)[i] << " "; 
	//cout << endl;
	//cout << "tolerance achived " << tolerance << endl;
	//cout << "number of iterations " << maxNumberOfIter << endl;
	return clock()-cl;
}

void SLAUSolver::initMatrix(){
	sparseAdapter.dimension = dimension;
	sparseAdapter.clear();
}

void SLAUSolver::createMatrix(){
	int *is = new int[sparseAdapter.getSize()];
	int *js = new int[sparseAdapter.getSize()];
	double *vs = new double[sparseAdapter.getSize()];
	sparseAdapter.getElements(is,js,vs);
	//for(int i=0;i<sparseAdapter.getSize();i++) cout << is[i]<< " " << js[i] << ">" << vs[i] << " "; cout << endl;
	//cout << "dimension: " << dimension << endl;
	//cout << "size: " << sparseAdapter.getSize() << endl;
	Coord_Mat_double MatrixC(dimension,dimension,sparseAdapter.getSize(),vs,is,js);
	if(Matrix != NULL) delete Matrix;
	Matrix = new CompCol_Mat_double(MatrixC);
	//writetxtfile_mat("temp.txt",*Matrix); 
	delete [] is;
	delete [] js;
	delete [] vs;
}

void SLAUSolver::setMatrixElement(int i,int j,double v){
	sparseAdapter.put(i,j,v);
}

double SLAUSolver::getMatrixElement(int i,int j){
	return sparseAdapter.get(i,j);
}

double SLAUSolver::addToMatrixElement(int i,int j,double v){
	return sparseAdapter.add(i,j,v);
}

void SLAUSolver::createPreconditioner(){
	if(Preconditioner != NULL) delete Preconditioner;
	Preconditioner = new ICPreconditioner_double(*Matrix);
}

//SLAUSolver::~SLAUSolver(){
//	if(solution!=NULL) delete solution;
//	if(vector_v!=NULL) delete vector_v;
//}
